<?php

namespace Ebuy\Review\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Ebuy\Review\Forms\Review as ReviewForm;
use Ebuy\Review\Http\Requests\ReviewRequest;
use Ebuy\Review\Interfaces\ReviewRepositoryInterface;
use Ebuy\Review\Repositories\Eloquent\Filters\ReviewResourceFilter;
use Ebuy\Review\Repositories\Eloquent\Presenters\ReviewListPresenter;

/**
 * Resource controller class for review.
 */
class ReviewResourceController extends BaseController
{

    /**
     * Initialize review resource controller.
     *
     *
     * @return null
     */
    public function __construct(ReviewRepositoryInterface $review)
    {
        parent::__construct();
        $this->form = ReviewForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('ebuy.review.modules'), 'review', guard_url('review'));
        $this->repository = $review;
    }

    /**
     * Display a list of review.
     *
     * @return Response
     */
    public function index(ReviewRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(ReviewResourceFilter::class)
            ->setPresenter(ReviewListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('review::review.names'))
            ->view('review::review.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display review.
     *
     * @param Request $request
     * @param Model   $review
     *
     * @return Response
     */
    public function show(ReviewRequest $request, ReviewRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('review::review.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('review::review.show')
            ->output();
    }

    /**
     * Show the form for creating a new review.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(ReviewRequest $request, ReviewRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('review::review.name'))
            ->view('review::review.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new review.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReviewRequest $request, ReviewRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('review::review.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('review/review/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/review/review'))
                ->redirect();
        }

    }

    /**
     * Show review for editing.
     *
     * @param Request $request
     * @param Model   $review
     *
     * @return Response
     */
    public function edit(ReviewRequest $request, ReviewRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('review::review.name'))
            ->view('review::review.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the review.
     *
     * @param Request $request
     * @param Model   $review
     *
     * @return Response
     */
    public function update(ReviewRequest $request, ReviewRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('review::review.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('review/review/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('review/review/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the review.
     *
     * @param Model   $review
     *
     * @return Response
     */
    public function destroy(ReviewRequest $request, ReviewRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('review::review.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('review/review/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('review/review/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
